const express = require('express');
const bodyParser = require('body-parser');
const morgan = require('morgan');
const ejs = require('ejs');
const app = express();
const http = require('http').createServer(app);
const WebSocket = require('ws');

const ipAddress = '192.168.43.121';
const port = 8080;

app.use(bodyParser.urlencoded({ extended: false }));
app.use(express.static('public'));
app.use(morgan('combined'));

app.set('view engine', 'ejs');

let gaz = '';
let apa = '';
let prez = '';
let locked = 1;
let opened = 0;

const clients = new Set();

app.get('/', (req, res) => {
	res.render('index', { key1, key2 });
});

app.get('/servo', (req, res) => {
	res.send({"locked": locked, "opened": opened});
});

app.post('/', (req, res) => {
	console.log(req.headers);
	console.log(req.body);
	gaz = Number(req.body.gaz);
	apa = Number(req.body.apa);
	prez = Number(req.body.prez);
	locked = Number(req.body.locked);
	opened = Number(req.body.opened);

	const data = JSON.stringify({ gaz, apa, prez, locked, opened});

	clients.forEach((client) => {
		client.send(data);
	});

	res.redirect('/');
});

http.listen(port, ipAddress, () => {
	console.log(`Server is running on http://${ipAddress}:${port}`);
});

const wss = new WebSocket.Server({ server: http });

wss.on('connection', (ws) => {
	clients.add(ws);
	ws.send(JSON.stringify({ gaz, apa, prez, locked, opened }));

	ws.on('close', () => {
		clients.delete(ws);
	});

	ws.on('message', (message) => {
		try {
			const data = JSON.parse(message);
			if (data.variable) {
				if (data.variable === 'open') {
					console.log('Received variable open');
					if (opened == 1)
						opened = 0;
					else
						opened = 1;
				} else if (data.variable === 'lock') {
					console.log('Received variable lock');
					if (locked == 1)
						locked = 0;
					else
						locked = 1;
				}

				console.log(JSON.stringify({ gaz, apa, prez, locked, opened }));
				clients.forEach((client) => {
					if (client.readyState === WebSocket.OPEN) {
						client.send(JSON.stringify({ gaz, apa, prez, locked, opened }));
					}
				});
			}
		} catch (error) {
			console.error('Error parsing message:', error);
		}
	});
});
