#include <Arduino.h>
#include <Keypad.h>
#include <Servo.h>

#define PREZENTA 11
#define SERVO 10
#define GAZ A0
#define APA A1

const byte ROWS = 4;
const byte COLS = 4;

byte rowPins[COLS] = {9, 8, 7, 6};
byte colPins[ROWS] = {5, 4, 3, 2};

char keys[ROWS][COLS] = {
	{'1','2','3','A'},
	{'4','5','6','B'},
	{'7','8','9','C'},
	{'*','0','#','D'}
};

Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );
Servo myservo;

const char passwd[] = "0000";
char buff[10];

int val_prez = 0;
int gas_value = 0;
int apa_value = 1024;
int last_apa = 0, last_gaz = 0;
int reason_servo = 1;
int locked = 1;
int opened = 0;
int opened_by_get = 0;

unsigned long previousMillis = 0;
const long interval = 10000;

void connectToWiFi() {
  Serial.print("AT+CWMODE=1\r\n");
  delay(1000);
  Serial.print("AT+CWJAP_CUR=\"AndroidAP3F6B\",\"shnz2318\"\r\n");
  delay(5000);
}

void sendPostRequest(int gaz, int apa, int prez, int opened, int locked) {
	char server[] = "192.168.43.121";

	char postData[50];
	char start_com[50];
	char start_send[30];
	postData[0] = '\0';
	start_com[0] = '\0';
	start_send[0] = '\0';
	sprintf(postData, "gaz=%d&apa=%d&prez=%d&opened=%d&locked=%d", gaz, apa, prez, opened, locked);

	char postRequest[200];
	sprintf(postRequest, "POST / HTTP/1.1\r\n");
	strcat(postRequest, "Host: ");
	strcat(postRequest, server);
	strcat(postRequest, ":8080\r\n");
	strcat(postRequest, "Content-Type: application/x-www-form-urlencoded\r\n");
	sprintf(postRequest + strlen(postRequest), "Content-Length: %d\r\n", strlen(postData));
	strcat(postRequest, "\r\n");
	strcat(postRequest, postData);
	sprintf(start_com, "AT+CIPSTART=\"TCP\",\"%s\",8080\r\n\r\n", server);
	Serial.print(start_com);
	delay(100);

	sprintf(start_send, "AT+CIPSEND=%d\r\n\r\n", strlen(postRequest));
	strcat(postRequest, "\r\n\r\n");
	Serial.print(start_send);
	delay(100);

	Serial.print(postRequest);
	delay(100);
	Serial.print("AT+CIPCLOSE\r\n\r\n");
}

void sendGetRequest() {
	char getRequest[] = "GET /servo HTTP/1.1\r\nHost: 192.168.43.121:8080\r\n\r\n\r\n";
	char server[] = "192.168.43.121";
	char start_com[50];
	char start_send[50];

	sprintf(start_com, "AT+CIPSTART=\"TCP\",\"%s\",8080\r\n\r\n", server);
	Serial.print(start_com);
	delay(100);

	sprintf(start_send, "AT+CIPSEND=%d\r\n\r\n", strlen(getRequest));
	Serial.print(start_send);
	delay(100);

	Serial.print(getRequest);
	delay(100);

	String resp = Serial.readString();

	Serial.print("AT+CIPCLOSE\r\n\r\n");

	char get_opened = resp[resp.length() - 2];
	char get_locked = resp[resp.length() - 13];
	if ((get_locked == '0' || get_locked == '1') && (get_opened == '0' || get_opened == '1')) {
		int to_open = get_opened - '0';
		int to_lock = get_locked - '0';
		
		if (to_lock != locked) {
			locked = to_lock;
		}

		if (to_open != opened) {
			if (to_open == 0) {
				if (!last_apa && !last_gaz) {
					opened_by_get = 1;
					if (myservo.read() != 0) {
						opened = 0;
						myservo.write(0);
						delay(1000);
					}
				}
			} else {
				opened_by_get = 1;
				if (myservo.read() != 90) {
					opened = 1;
					myservo.write(90);
					delay(1000);
				}
			}
		}
	}
}

void setup()
{
	Serial.begin(9600);
	pinMode(PREZENTA, INPUT);
	pinMode(GAZ, INPUT);
	pinMode(APA, INPUT);
	myservo.attach(SERVO);
	myservo.write(0);
	buff[0] = '\0';
	delay(10000);
	connectToWiFi();
}

void loop()
{
	int current_apa = 0, ok = 0, gaz = 0;
	unsigned long currentMillis = millis();
	if (currentMillis - previousMillis >= interval) {
		if (opened_by_get)
			opened_by_get = 0;

		if (strlen(buff) == 0) {
			previousMillis = currentMillis;
			sendGetRequest();
		}

		if (strlen(buff) != 0 && currentMillis - previousMillis >= 1.5*interval) {
			previousMillis = currentMillis;
			buff[0] = '\0';
			sendGetRequest();
		}
	}

	gas_value = analogRead(GAZ);
	apa_value = analogRead(APA);
	int aux = digitalRead(PREZENTA);
	char key = keypad.getKey();

	if(gas_value > 250) {
		gaz = 1;
	} else {
		gaz = 0;
	}

	if (apa_value > 750) {
		current_apa = 0;
	} else {
		if(apa_value > 400) {
			current_apa = 1;
		} else {
			current_apa = 3;
		}
	}

	if(last_apa != current_apa) {
		last_apa = current_apa;
		ok = 1;
	}
	
	if(last_gaz != gaz) {
		last_gaz = gaz;
		ok = 1;
	}

	if (aux != val_prez) {
		val_prez = aux;
		ok = 1;
	}

	if (key) {
		if (key == '#') {
			if(!strcmp(buff, passwd)) {
				locked = 0;
				ok = 1;
			}
			buff[0] = '\0';
		} else {
			if (key == '*') {
				if(!strcmp(buff, passwd)) {
					locked = 1;
					ok = 1;
				}
				buff[0] = '\0';
			} else {
				if (strlen(buff) < 10)
					sprintf(buff, "%s%c", buff, key);
				else {
					buff[0] = key;
					buff[1] = '\0';
				}
			}
		}
	}

	if (gaz || current_apa == 3 || (!locked && val_prez && !opened_by_get)) {
		if (myservo.read() != 90) {
			opened = 1;
			ok = 1;
			myservo.write(90);
			delay(1000);
		}
	}

	if (!gaz && !current_apa) {
		if (locked) {
			if (myservo.read() != 0 && !opened_by_get) {
				opened = 0;
				ok = 1;
				myservo.write(0);
				delay(1000);
			}
		} else {
			if (!val_prez) {
				if (myservo.read() != 0 && !opened_by_get) {
					opened = 0;
					ok = 1;
					myservo.write(0);
					delay(1000);
				}
			}
		}
	}

	if(ok) {
		// char postData[50];
		// sprintf(postData, "gaz=%d&apa=%d&prez=%d&opened=%d&locked=%d", gaz, current_apa, val_prez, opened, locked);
		// Serial.println(postData);
		sendPostRequest(gaz, current_apa, val_prez, opened, locked);
	}
}